/**
 * OW2 Util
 * Copyright (C) 2010-2011 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: StreamReader.java 5691 2011-01-05 22:47:22Z loris $
 * --------------------------------------------------------------------------
 */
package org.ow2.util.protocol.impl;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.util.file.FileUtils;

/**
 * @author Loris Bouzonnet
 */
public class StreamReader implements Runnable {

    /**
     * Logger.
     */
    private static Logger     logger =
                                             Logger.getLogger(StreamReader.class.getName());

    private final InputStream inputStream;

    private final String      type;

    private final boolean     error;

    private final OutputStream outputStream;

    /**
     * Reads the stream with the logger.
     * @param inputStream
     * @param type
     * @param error true if the stream is related to an error
     */
    public StreamReader(final InputStream inputStream, final String type,
            final boolean error) {
        this.inputStream = inputStream;
        this.error = error;
        this.type = type;
        this.outputStream = null;
    }

    /**
     * Reads the stream into an other stream.
     * @param inputStream
     * @param outputStream
     */
    public StreamReader(final InputStream inputStream, final OutputStream outputStream) {
        this.inputStream = inputStream;
        this.outputStream = outputStream;
        this.error = false;
        this.type = null;
    }

    public void run() {

        if (this.outputStream == null) {

            // Log -> prefer reader
            // TODO: Which charset?
            final InputStreamReader isr = new InputStreamReader(this.inputStream);
            final BufferedReader br = new BufferedReader(isr);

            String line = null;
            try {
                while ((line = br.readLine()) != null) {

                    if (this.error) {
                        logger.severe(this.type + "> " + line);
                    } else {
                        logger.info(this.type + "> " + line);
                    }
                }
            } catch (final IOException e) {
                logger.log(Level.SEVERE, "Error when reading a new line", e);
            } finally {
                FileUtils.close(br, this.inputStream);
            }
        } else {

            // Redirect -> keep bytes
            int readBytes;
            final byte[] bytes = new byte[512];
            try {
                while ((readBytes = this.inputStream.read(bytes)) > 0) {
                    this.outputStream.write(bytes, 0, readBytes);
                }
            } catch (final IOException e) {
                logger.log(Level.SEVERE, "Error when reading/writing some bytes", e);
            }
        }

    }

}
